import { Component, createEffect, createSignal, on } from 'solid-js';

import styles from './Page.module.css';
import Navigation from '../Navigation';

function murmurHash3(string: string) {
	let i = 0, hash: number;
	for (hash = 1779033703 ^ string.length; i < string.length; i++) {
		const bitwise_xor_from_character = hash ^ string.charCodeAt(i);
		hash = Math.imul(bitwise_xor_from_character, 3432918353);
		hash = hash << 13 | hash >>> 19;
	}
	return () => {
		// Return the hash that you can use as a seed
		hash = Math.imul(hash ^ (hash >>> 16), 2246822507);
		hash = Math.imul(hash ^ (hash >>> 13), 3266489909);
		return (hash ^= hash >>> 16) >>> 0;
	};
}

function simpleFastCounter32(seed_1: number, seed_2: number, seed_3: number, seed_4: number) {
	return () => {
		seed_1 >>>= 0;
		seed_2 >>>= 0;
		seed_3 >>>= 0;
		seed_4 >>>= 0;
		let cast32 = (seed_1 + seed_2) | 0;
		seed_1 = seed_2 ^ seed_2 >>> 9;
		seed_2 = seed_3 + (seed_3 << 3) | 0;
		seed_3 = (seed_3 << 21 | seed_3 >>> 11);
		seed_4 = seed_4 + 1 | 0;
		cast32 = cast32 + seed_4 | 0;
		seed_3 = seed_3 + cast32 | 0;
		return (cast32 >>> 0) / 4294967296;
	};
}

const StrNoise: Component = () => {
	const [output, setOutput] = createSignal('');
	const [input, setInput] = createSignal('');
	const [noiseLevel, setNoiseLevel] = createSignal(10);
	const [seed, setSeed] = createSignal('0');

	const [shuffle, setShuffle] = createSignal(true);
	const [randomize, setRandomize] = createSignal(true);
	const [swapAdjacent, setSwapAdjancent] = createSignal(true);

	createEffect(on([input, noiseLevel, seed, shuffle, randomize, swapAdjacent], () => {
		const s = murmurHash3(seed());
		const generator = simpleFastCounter32(s(), s(), s(), s());

		const types = [];
		if (shuffle()) types.push('shuffle');
		if (randomize()) types.push('randomize');
		if (swapAdjacent()) types.push('swap_adjacent');

		const arr = input().split('');
		const its = noiseLevel() / 100 * arr.length * 2;
		for (let i = 0; i < its; i++) {
			const type = types[Math.floor(generator() * types.length)];

			if (type == 'shuffle') {
				const a = Math.floor(generator() * arr.length);
				const b = Math.floor(generator() * arr.length);
				[arr[a], arr[b]] = [arr[b], arr[a]];
			} else if (type == 'randomize') {
				// TODO
			} else if (type == 'swap_adjacent') {
				const a = Math.floor(generator() * (arr.length - 1));
				[arr[a], arr[a + 1]] = [arr[a + 1], arr[a]];
			}
		}
		setOutput(arr.join(''));
	}));

	let outputField: HTMLInputElement | undefined;

	const copy = async () => {
		outputField!.select();
		document.execCommand('copy');
	};

	return (
		<div class={styles.page}>
			<h1>String Noise</h1>
			<h2>Input</h2>
			<textarea value={input()} oninput={e => setInput(e.currentTarget.value)} />
			<h2>Noise Level</h2>
			<input type={'range'} min={0} max={100} value={noiseLevel()} oninput={e => setNoiseLevel(parseInt(e.currentTarget.value))} />
			<h2>Noise Types</h2>
			<div class={styles.checkbox}>
				<input type={'checkbox'} checked={shuffle()} onchange={e => setShuffle(e.currentTarget.checked)} />
				<label>Shuffle</label>
			</div>
			<div class={styles.checkbox}>
				<input type={'checkbox'} checked={randomize()} onchange={e => setRandomize(e.currentTarget.checked)} />
				<label>Randomize</label>
			</div>
			<div class={styles.checkbox}>
				<input type={'checkbox'} checked={swapAdjacent()} onchange={e => setSwapAdjancent(e.currentTarget.checked)} />
				<label>Swap Adjacent</label>
			</div>
			<h2>Seed</h2>
			<input min={0} max={100} value={seed()} onInput={e => setSeed(e.currentTarget.value)} />
			<h2>Output</h2>
			<input value={output()} ref={outputField} />
			<button onclick={copy}>Copy Output</button>
			<h2>All Tools</h2>
			<Navigation />
		</div>
	);
};

export default StrNoise;
